import { useReducer, useMemo } from "react";
import createUseContext from "constate";
import { tryCreateDevToolsLogger } from "./devtools";
export function useMethods(initialState, methods) {
    const reducer = useMemo(() => {
        const logger = tryCreateDevToolsLogger();
        return (state, action) => {
            const nextState = methods[action.type](state)(...action.payload);
            if (logger) {
                logger(action, nextState);
            }
            return nextState;
        };
    }, []);
    const [state, dispatch] = useReducer(reducer, initialState);
    const methodsOut = useMemo(() => {
        const methodsOut = {};
        Object.keys(methods).forEach(type => {
            methodsOut[type] = (...payload) => {
                dispatch({
                    type,
                    payload
                });
            };
        });
        return methodsOut;
    }, []);
    return { state, methods: methodsOut };
}
export const createUseStore = (defaultState, methodsIn) => createUseContext(({ initialState }) => useMethods(initialState || defaultState, methodsIn), 
// https://kentcdodds.com/blog/always-use-memo-your-context-value
// Pass to context not a new object every time
({ state }) => [state]);
